/* SECU-3  - An open source, free engine control unit
   Copyright (C) 2007 Alexey A. Shabelnikov. Ukraine, Gorlovka

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   contacts:
              http://secu-3.org
              email: shabelnikov@secu-3.org
*/

#include "stdafx.h"
#include <map>
#include "FirmwareDataMediator.h"
#include "CRC16.h"
#include "ControlApp.h"    //should be removed - it is nearly unnecessary!

//-----@Add_serge_5518------
#if defined(USE_FOR_API_BUILDER)
#include "MathHelpers.h"
#include <Dialogs.hpp> // for ShowMessage

#else
#include "common/MathHelpers.h"
#endif
//-end-@

#include "FirmwareMapsDataHolder.h"
#include "PlatformParamHolder.h"
#include "SECU3IO.h"
#include "SECU3ParametersDef.h"
#include "SECU3TablesDef.h"
#include "ufcodes.h"

//-----@Add_serge_5518------
#if defined(USE_FOR_API_BUILDER)
#define ASSERT(a) _ASSERTE((a))
#endif
//-end-@

//   -  
#pragma pack(1)

using namespace SECU3IO;
using namespace SECU3IO::SECU3Types;

#define KC_ATTENUATOR_LOOKUP_TABLE_SIZE  128
#define FW_SIGNATURE_INFO_SIZE           48
#define COIL_ON_TIME_LOOKUP_TABLE_SIZE   32
#define THERMISTOR_LOOKUP_TABLE_SIZE     16

//-----@Add_serge_5518------
#if defined(USE_FOR_API_BUILDER)
#define _BIGEND16(v) ((size_t)((((v) >> 8) & 0x00FF) | (((v) << 8) & 0xFF00)))
//-----@Add_serge_5518------
//  CAST_CDDATA [BCC32 Warning]: W8012 Comparing signed and unsigned values

#else
#define _BIGEND16(v) ((((v) >> 8) & 0x00FF) | (((v) << 8) & 0xFF00))
#endif
//-end-@

//We rely that size member is valid in all versions of cd_data_t structure and use it to determine
//version of the structure.
#define CAST_CDDATA(ptr, mbr) ((_BIGEND16((ptr)->size) == sizeof(cd_datav0_t)) ? (((cd_datav0_t*)( ((((_uchar*)(ptr)) + sizeof(cd_data_t)) - sizeof(cd_datav0_t)) ))->mbr) : (((cd_data_t*)(ptr))->mbr))

#define IOREM_MAJ_VER(v) (((v) >> 4) & 0xf)

#define IOREM_SLOTS_NUM(ptr) (size_t)(((IOREM_MAJ_VER(CAST_CDDATA(ptr, iorem.version)) == 0) ? IOREM_SLOTSv0 : IOREM_SLOTS))
#define IOREM_PLUGS_NUM(ptr) (size_t)(((IOREM_MAJ_VER(CAST_CDDATA(ptr, iorem.version)) == 0) ? IOREM_PLUGSv0 : IOREM_PLUGS))

#define IOP_COUNT_NUM(ptr) ((IOREM_MAJ_VER(CAST_CDDATA(ptr, iorem.version)) == 0) ? IOP_COUNTv0 : IOP_COUNT)
#define IOS_COUNT_NUM(ptr) ((IOREM_MAJ_VER(CAST_CDDATA(ptr, iorem.version)) == 0) ? IOS_COUNTv0 : IOS_COUNT)

//--------------------------For iorem_slots_t V0.0----------------------------
//See also FirmwareDataMediator.h
#define IOREM_SLOTSv0 10         // Number of slots used for I/O remapping
#define IOREM_PLUGSv0 16         // Number of plugs used in I/O remapping

//Describes all data related to I/O remapping
typedef struct iorem_slotsv0_t
{
 _uchar size;                    // size of this structure
 _uchar version;                 // A reserved byte = 0

 //Dirty hack. This version of struct actually has no i_slotsi member,
 //it is necessary for CAST_CDDATA
union {
 _fnptr_t i_slots[IOREM_SLOTSv0];// initialization slots
 _fnptr_t i_slotsi[IOREM_SLOTSv0];// initialization slots (stub)
};

 //Dirty hack. This version of struct actually has no v_slotsi member,
 //it is necessary for CAST_CDDATA
union {
 _fnptr_t v_slots[IOREM_SLOTSv0];// data slots
 _fnptr_t v_slotsi[IOREM_SLOTSv0];// data slots           (stub)
};

 _fnptr_t i_plugs[IOREM_PLUGSv0];// initialization plugs
 _fnptr_t v_plugs[IOREM_PLUGSv0];// data plugs
 _fnptr_t s_stub;                // special pointer used as stub
 _fnptr_t g_stub;                // reserved = 0
}iorem_slotsv0_t;

//Describes data stored directly in the firmware code
typedef struct cd_datav0_t
{
 // Arrays which are used for I/O remapping. Some arrays are "slots", some are "plugs"
 iorem_slotsv0_t iorem;

 //holds flags which give information about options were used to build firmware
 //(           )
 _ulong config;

 _uchar reserved[2];             //two reserved bytes

 _uchar size;                    // size of this structure
}cd_datav0_t;

//------------------------For iorem_slots_t V1.0----------------------------
#define IOREM_SLOTS 16           // Number of slots used for I/O remapping
#define IOREM_PLUGS 32           // Number of plugs used in I/O remapping

//Describes all data related to I/O remapping
typedef struct iorem_slots_t
{
 _uint size;                     // size of this structure
 _uchar version;                 // A reserved byte
 _fnptr_t i_slots[IOREM_SLOTS];  // initialization slots
 _fnptr_t i_slotsi[IOREM_SLOTS]; // initialization slots (inverted)
 _fnptr_t v_slots[IOREM_SLOTS];  // data slots
 _fnptr_t v_slotsi[IOREM_SLOTS]; // data slots           (inverted)
 _fnptr_t i_plugs[IOREM_PLUGS];  // initialization plugs
 _fnptr_t v_plugs[IOREM_PLUGS];  // data plugs
 _fnptr_t s_stub;                // special pointer used as stub (for outputs)
 _fnptr_t g_stub;                // special pointer used as stub (for inputs)
}iorem_slots_t;

//Describes data stored directly in the firmware code
typedef struct cd_data_t
{
 // Arrays which are used for I/O remapping. Some arrays are "slots", some are "plugs"
 iorem_slots_t iorem;

 //holds flags which give information about options were used to build firmware
 //(           )
 _ulong config;

 _uchar reserved;                //A reserved byte

 _uint size;                     // size of this structure (2 bytes, big-endian format)
}cd_data_t;

//     
typedef struct
{
 //  
 _uchar fw_signature_info[FW_SIGNATURE_INFO_SIZE];

 //   (  ).
 _uchar attenuator_table[KC_ATTENUATOR_LOOKUP_TABLE_SIZE];

 //lookup table containing accumulation time for ignition coils (dependence from voltage)
 //       (  )
 _uint coil_on_time[COIL_ON_TIME_LOOKUP_TABLE_SIZE];

 //used for checking compatibility with management software. Holds size of all data stored in the firmware.
 //Includes CRC size also.
 _uint fw_data_size;

 //reserved 32-bit value
 _ulong reserv32;

 //Coolant temperature sensor lookup table (      )
 _int cts_curve[THERMISTOR_LOOKUP_TABLE_SIZE];
 //Related voltage limits
 _uint cts_vl_begin;
 _uint cts_vl_end;

 //       
 //      .    
 // ,    .
 _uchar reserved[22];
}fw_ex_data_t;

//Describes all data residing in the firmware
typedef struct fw_data_t
{
 fw_ex_data_t exdata;                    // Additional data in the firmware
 params_t     def_param;                 // Reserve parameters (loaded when instance in EEPROM is broken)
 f_data_t     tables[TABLES_NUMBER];     // Array of tables of advance angle
 _uint        code_crc;                  // Check sum of the whole firmware (except this check sum and boot loader)
}fw_data_t;


class CFirmwareDataMediator::LocInfoProvider
 {
  public:
   LocInfoProvider(const PPFlashParam& i_fpp)
   : CODE_CRC_SIZE(sizeof(fw_data_t().code_crc))
   , BOOT_START(i_fpp.m_app_section_size)
   , CODE_SIZE(BOOT_START - CODE_CRC_SIZE)
   , FIRMWARE_DATA_START(BOOT_START - sizeof(fw_data_t))
   , FIRMWARE_DATA_SIZE(sizeof(fw_data_t) - CODE_CRC_SIZE)
   {
    //empty
   }

   //size of check sum in bytes
   const size_t CODE_CRC_SIZE;
   //starting address of boot loader section
   const size_t BOOT_START;
   //       
   const size_t CODE_SIZE;
   //      
   const size_t FIRMWARE_DATA_START;
   //         
   const size_t FIRMWARE_DATA_SIZE;
 };

CFirmwareDataMediator::CFirmwareDataMediator(const PPFlashParam& i_fpp)
: m_firmware_size(i_fpp.m_total_size)
, m_is_opened(false)
, m_fpp(new PPFlashParam(i_fpp))
, m_lip(new LocInfoProvider(i_fpp))
, mp_cddata(NULL)
{
 mp_bytes_active = new BYTE[m_fpp->m_total_size + 1];
 ASSERT(mp_bytes_active);
 mp_bytes_original = new BYTE[m_fpp->m_total_size + 1];
 ASSERT(mp_bytes_original);

 memset(mp_bytes_active, 0x00, m_firmware_size);
 memset(mp_bytes_original, 0x00, m_firmware_size);
}

//-----@Add_serge_5518------
#if defined(USE_FOR_API_BUILDER)
CFirmwareDataMediator::~CFirmwareDataMediator(void)
#else
CFirmwareDataMediator::~CFirmwareDataMediator()
#endif//-end-@
{
 delete[] mp_bytes_active;
 delete[] mp_bytes_original;
}

//        
//io_data -   
void CFirmwareDataMediator::CalculateAndPlaceFirmwareCRC(BYTE* io_data)
{
 _uint crc = crc16(io_data, m_lip->CODE_SIZE);
 fw_data_t* p_fd = (fw_data_t*)(&io_data[m_lip->FIRMWARE_DATA_START]);
 p_fd->code_crc = crc; //  
}

bool CFirmwareDataMediator::CheckCompatibility(const BYTE* i_data, const PPFlashParam* ip_fpp /*= NULL*/) const
{
 if (!ip_fpp)
  ip_fpp = m_fpp.get(); //use information form current firmware

 LocInfoProvider lip(*ip_fpp);

 bool compatible = true;

 fw_data_t* p_fd = (fw_data_t*)&i_data[lip.FIRMWARE_DATA_START];

 //size of firmware data
 int sizeoffd = p_fd->def_param.crc;

 //size of code area data (must be valid if def_param.crc > 0)
 _uint sizeofcd = _BIGEND16(*(((_uint*)&i_data[lip.FIRMWARE_DATA_START])-1));

 cd_data_t* p_cd = (cd_data_t*)&i_data[lip.FIRMWARE_DATA_START - sizeof(cd_data_t)];
 size_t iorem_struct_size = (sizeofcd == sizeof(cd_datav0_t)) ? sizeof(iorem_slotsv0_t) : sizeof(iorem_slots_t);

 //-----@Add_serge_5518------
#if defined(USE_FOR_API_BUILDER)
 //[BCC32 Warning] W8012 Comparing signed and unsigned values
 //   CAST_CDDATA   size_t
 if ((sizeof(fw_data_t)) != p_fd->exdata.fw_data_size ||
  ((sizeoffd > 0) && sizeoffd != p_fd->exdata.fw_data_size) ||
  ((sizeoffd > 0) && (sizeofcd != sizeof(cd_datav0_t) && sizeofcd != sizeof(cd_data_t))) ||
  ((sizeoffd > 0) && (((size_t)CAST_CDDATA(p_cd, iorem.size)) != iorem_struct_size)) ||
  //supported major versions: 0, 1
  ((sizeoffd > 0) && (IOREM_MAJ_VER(CAST_CDDATA(p_cd, iorem.version)) != 0 &&
     IOREM_MAJ_VER(CAST_CDDATA(p_cd, iorem.version)) != 1)))
  compatible = false;
#else
 if ((sizeof(fw_data_t)) != p_fd->exdata.fw_data_size ||
  ((sizeoffd > 0) && sizeoffd != p_fd->exdata.fw_data_size) ||
  ((sizeoffd > 0) && (sizeofcd != sizeof(cd_datav0_t) && sizeofcd != sizeof(cd_data_t))) ||
  ((sizeoffd > 0) && CAST_CDDATA(p_cd, iorem.size) != iorem_struct_size) ||
  //supported major versions: 0, 1
  ((sizeoffd > 0) && (IOREM_MAJ_VER(CAST_CDDATA(p_cd, iorem.version)) != 0 &&
     IOREM_MAJ_VER(CAST_CDDATA(p_cd, iorem.version)) != 1)))
  compatible = false;
#endif
//-end-@

 return compatible;
}

void CFirmwareDataMediator::LoadBytes(const BYTE* i_bytes)
{
 ASSERT(i_bytes);
 memset(mp_bytes_active,0x00,m_firmware_size);
 memset(mp_bytes_original,0x00,m_firmware_size);

 memcpy(mp_bytes_active,i_bytes,m_firmware_size);
 memcpy(mp_bytes_original,i_bytes,m_firmware_size);

 _FindCodeData(); //find data residing directly in the code
 m_is_opened = true;
}

void CFirmwareDataMediator::StoreBytes(BYTE* o_bytes)
{
 ASSERT(o_bytes);
 memcpy(o_bytes,mp_bytes_active,m_firmware_size);
}

bool CFirmwareDataMediator::IsModified(void)
{
 return (0 != memcmp(mp_bytes_active,mp_bytes_original,m_firmware_size));
}

void CFirmwareDataMediator::ResetModified(void)
{
 memcpy(mp_bytes_original,mp_bytes_active,m_firmware_size);
}

_TSTRING CFirmwareDataMediator::GetSignatureInfo(void)
{
 char raw_string[256];
 memset(raw_string,0,FW_SIGNATURE_INFO_SIZE+1);
 fw_data_t* p_fd = (fw_data_t*)(&mp_bytes_active[m_lip->FIRMWARE_DATA_START]); 
 memcpy(raw_string, &p_fd->exdata.fw_signature_info, FW_SIGNATURE_INFO_SIZE);
 TCHAR string[256];
 OemToChar(raw_string, string);
 return _TSTRING(string);
}

void CFirmwareDataMediator::SetSignatureInfo(const _TSTRING& i_string)
{
 _TSTRING str = i_string;
 //string must be fixed length
 while(str.length() < FW_SIGNATURE_INFO_SIZE)
  str+=' ';
 char raw_string[256];
 memset(raw_string, 0, FW_SIGNATURE_INFO_SIZE+1);
 fw_data_t* p_fd = (fw_data_t*)(&mp_bytes_active[m_lip->FIRMWARE_DATA_START]); 
 CharToOem(str.c_str(), raw_string);
 memcpy(&p_fd->exdata.fw_signature_info, raw_string, FW_SIGNATURE_INFO_SIZE);
}

DWORD CFirmwareDataMediator::GetFWOptions(void)
{
 if (mp_cddata)
  return CAST_CDDATA(mp_cddata, config);
 else
 {
  //there is no such data in this firmware, then we have to use old place
  fw_data_t* p_fd = (fw_data_t*)(&mp_bytes_active[m_lip->FIRMWARE_DATA_START]); 
  return p_fd->exdata.reserv32; 
 }
}

void CFirmwareDataMediator::GetStartMap(int i_index,float* o_values, bool i_original /* = false */)
{
 BYTE* p_bytes = NULL;
 f_data_t* p_maps = NULL;
 ASSERT(o_values);

 if (i_original)
  p_bytes = mp_bytes_original;
 else
  p_bytes = mp_bytes_active;

 //     
 fw_data_t* p_fd = (fw_data_t*)(p_bytes + m_lip->FIRMWARE_DATA_START);

 for (int i = 0; i < F_STR_POINTS; i++ )
  o_values[i] = ((float)p_fd->tables[i_index].f_str[i]) / AA_MAPS_M_FACTOR;
}

void CFirmwareDataMediator::SetStartMap(int i_index,const float* i_values)
{
 BYTE* p_bytes = NULL;
 f_data_t* p_maps = NULL;
 ASSERT(i_values);

 p_bytes = mp_bytes_active;

 //     
 fw_data_t* p_fd = (fw_data_t*)(p_bytes + m_lip->FIRMWARE_DATA_START);

 for (int i = 0; i < F_STR_POINTS; i++ )
  p_fd->tables[i_index].f_str[i] = MathHelpers::Round((i_values[i]*AA_MAPS_M_FACTOR));
}

void CFirmwareDataMediator::GetIdleMap(int i_index,float* o_values, bool i_original /* = false */)
{
 BYTE* p_bytes = NULL;
 f_data_t* p_maps = NULL;
 ASSERT(o_values);

 if (i_original)
  p_bytes = mp_bytes_original;
 else
  p_bytes = mp_bytes_active;

 //     
 fw_data_t* p_fd = (fw_data_t*)(p_bytes + m_lip->FIRMWARE_DATA_START);

 for (int i = 0; i < F_IDL_POINTS; i++ )
  o_values[i] = ((float)p_fd->tables[i_index].f_idl[i]) / AA_MAPS_M_FACTOR;
}

void CFirmwareDataMediator::SetIdleMap(int i_index,const float* i_values)
{
 BYTE* p_bytes = NULL;
 f_data_t* p_maps = NULL;
 ASSERT(i_values);

 p_bytes = mp_bytes_active;

 //     
 fw_data_t* p_fd = (fw_data_t*)(p_bytes + m_lip->FIRMWARE_DATA_START);

 for (int i = 0; i < F_IDL_POINTS; i++ )
  p_fd->tables[i_index].f_idl[i] = MathHelpers::Round((i_values[i]*AA_MAPS_M_FACTOR));
}

std::vector<_TSTRING> CFirmwareDataMediator::GetFunctionsSetNames(void)
{
 std::vector<_TSTRING> names(TABLES_NUMBER);

 BYTE* p_bytes = NULL;
 f_data_t* p_maps = NULL;

 p_bytes = mp_bytes_active;

 //     
 fw_data_t* p_fd = (fw_data_t*)(p_bytes + m_lip->FIRMWARE_DATA_START);

 for (int i = 0; i < TABLES_NUMBER; i++)
 {
  char raw_string[256];
  memset(raw_string, 0, F_NAME_SIZE+1);
  BYTE* addr = p_fd->tables[i].name;
  memcpy(raw_string, addr, F_NAME_SIZE);
  TCHAR string[256];
  OemToChar(raw_string,string);
  names[i] = string;
 }

return names;
}

void CFirmwareDataMediator::SetFunctionsSetName(int i_index, _TSTRING i_new_name)
{
 if (i_index >= TABLES_NUMBER)
  return;

 BYTE* p_bytes = NULL;
 f_data_t* p_maps = NULL;

 p_bytes = mp_bytes_active;

 //     
 fw_data_t* p_fd = (fw_data_t*)(p_bytes + m_lip->FIRMWARE_DATA_START);

 //  ,       F_NAME_SIZE,
 //   
 i_new_name+= _T("                ");

 char raw_string[256];
 memset(raw_string, 0, F_NAME_SIZE+1);
 CharToOem(i_new_name.c_str(), raw_string);
 memcpy(p_fd->tables[i_index].name, raw_string, F_NAME_SIZE);
}

void CFirmwareDataMediator::SetFWFileName(const _TSTRING i_fw_file_name)
{
 m_fw_file_name = i_fw_file_name;
}

_TSTRING CFirmwareDataMediator::GetFWFileName(void)
{
 return m_fw_file_name;
}

unsigned int CFirmwareDataMediator::CalculateCRC16OfActiveFirmware(void)
{
 _uint crc_16 = crc16(mp_bytes_active, m_lip->CODE_SIZE);
 return crc_16;
}

unsigned int CFirmwareDataMediator::GetCRC16StoredInActiveFirmware(void)
{
 fw_data_t* p_fd = (fw_data_t*)(&mp_bytes_active[m_lip->FIRMWARE_DATA_START]);
 return p_fd->code_crc;
}

void CFirmwareDataMediator::CalculateAndPlaceFirmwareCRC(void)
{
 CalculateAndPlaceFirmwareCRC(mp_bytes_active);
}

void CFirmwareDataMediator::LoadDataBytesFromAnotherFirmware(const BYTE* i_source_bytes, const PPFlashParam* ip_fpp /*= NULL*/)
{
 if (false==IsLoaded())
  return; // ...

 //Now we need to load data stored in the code area to the current firmware
 LoadCodeData(i_source_bytes, (ip_fpp ? ip_fpp : m_fpp.get())->m_app_section_size);

 fw_data_t* p_fd = (fw_data_t*)(&mp_bytes_active[m_lip->FIRMWARE_DATA_START]);
 size_t dataSize = p_fd->def_param.crc;
 _ulong oldFWOpt = p_fd->exdata.reserv32;
 _uint oldFWCRC  = p_fd->code_crc;

 if (ip_fpp)
 {
  LocInfoProvider lip(*ip_fpp);

  //shrink size using value specified in def_param.crc if def_param.crc contain a valid value
  size_t start_d = (0==dataSize) ? m_lip->FIRMWARE_DATA_START : (m_lip->BOOT_START - dataSize);
  size_t start_s = (0==dataSize) ? lip.FIRMWARE_DATA_START : (lip.BOOT_START - dataSize);
  size_t size = (0==dataSize) ? m_lip->FIRMWARE_DATA_SIZE : dataSize; 
  memcpy(mp_bytes_active + start_d, i_source_bytes + start_s, size);
 }
 else
 {
  //shrink size using value specified in def_param.crc if def_param.crc contain a valid value
  size_t start = (0==dataSize) ? m_lip->FIRMWARE_DATA_START : (m_lip->BOOT_START - dataSize);
  size_t size = (0==dataSize) ? m_lip->FIRMWARE_DATA_SIZE : dataSize; 
  memcpy(mp_bytes_active + start, i_source_bytes + start, size);  
 }
 // def_param.crc  ,    .
 //       ,     exdata.reserv32
 // ,    .
 p_fd->def_param.crc = dataSize;
 if (0==dataSize)
  p_fd->exdata.reserv32 = oldFWOpt;

 //    
 p_fd->code_crc = oldFWCRC;

 _FindCodeData();
}

void CFirmwareDataMediator::LoadDefParametersFromBuffer(const BYTE* i_source_bytes)
{
 if (false==IsLoaded())
  return; // ...
 fw_data_t* p_fd = (fw_data_t*)(&mp_bytes_active[m_lip->FIRMWARE_DATA_START]);
 _uint fwd_size = p_fd->def_param.crc; //save
 memcpy(&p_fd->def_param, i_source_bytes, sizeof(params_t));
 p_fd->def_param.crc = fwd_size; //restore
 _FindCodeData(); //find data residing directly in the code
}

void CFirmwareDataMediator::GetWorkMap(int i_index, float* o_values, bool i_original /* = false*/)
{
 BYTE* p_bytes = NULL;
 f_data_t* p_maps = NULL;
 ASSERT(o_values);

 if (i_original)
  p_bytes = mp_bytes_original;
 else
  p_bytes = mp_bytes_active;

 //     
 fw_data_t* p_fd = (fw_data_t*)(&p_bytes[m_lip->FIRMWARE_DATA_START]);

 for (int i = 0; i < (F_WRK_POINTS_F * F_WRK_POINTS_L); i++ )
 {
  _char *p = &(p_fd->tables[i_index].f_wrk[0][0]);
  o_values[i] = ((float) *(p + i)) / AA_MAPS_M_FACTOR;
 }
}

void CFirmwareDataMediator::SetWorkMap(int i_index, const float* i_values)
{
 BYTE* p_bytes = NULL;
 f_data_t* p_maps = NULL;
 ASSERT(i_values);

 p_bytes = mp_bytes_active;

 //     
 fw_data_t* p_fd = (fw_data_t*)(&p_bytes[m_lip->FIRMWARE_DATA_START]);

 for (int i = 0; i < (F_WRK_POINTS_F * F_WRK_POINTS_L); i++ )
 {
  _char *p = &(p_fd->tables[i_index].f_wrk[0][0]);
  *(p + i) = MathHelpers::Round((i_values[i]*AA_MAPS_M_FACTOR));
 }
}

void CFirmwareDataMediator::GetTempMap(int i_index,float* o_values, bool i_original /* = false */)
{
 BYTE* p_bytes = NULL;
 f_data_t* p_maps = NULL;
 ASSERT(o_values);

 if (i_original)
  p_bytes = mp_bytes_original;
 else
  p_bytes = mp_bytes_active;

 //     
 fw_data_t* p_fd = (fw_data_t*)(&p_bytes[m_lip->FIRMWARE_DATA_START]);

 for (int i = 0; i < F_TMP_POINTS; i++ )
  o_values[i] = ((float)p_fd->tables[i_index].f_tmp[i]) / AA_MAPS_M_FACTOR;
}

void CFirmwareDataMediator::SetTempMap(int i_index,const float* i_values)
{
 BYTE* p_bytes = NULL;
 f_data_t* p_maps = NULL;
 ASSERT(i_values);

 p_bytes = mp_bytes_active;

 //     
 fw_data_t* p_fd = (fw_data_t*)(&p_bytes[m_lip->FIRMWARE_DATA_START]);

 for (int i = 0; i < F_TMP_POINTS; i++ )
  p_fd->tables[i_index].f_tmp[i] = MathHelpers::Round((i_values[i]*AA_MAPS_M_FACTOR));
}

bool CFirmwareDataMediator::SetDefParamValues(BYTE i_descriptor, const void* i_values)
{
 using namespace SECU3IO;

 BYTE* p_bytes = NULL;
 params_t* p_params = NULL;
 p_bytes = mp_bytes_active;
 //    
 p_params = &((fw_data_t*)(&p_bytes[m_lip->FIRMWARE_DATA_START]))->def_param;

 //TODO: Remove these copy/paste! (conversions should be implemented in functions, as certain class)
 switch(i_descriptor)
 {
  case TEMPER_PAR:
   {
    TemperPar* p_in = (TemperPar*)i_values;
    p_params->tmp_use  = p_in->tmp_use;
    p_params->vent_pwm = p_in->vent_pwm;
    p_params->cts_use_map = p_in->cts_use_map;
    p_params->vent_on  = MathHelpers::Round(p_in->vent_on * TEMP_PHYSICAL_MAGNITUDE_MULTIPLAYER);
    p_params->vent_off = MathHelpers::Round(p_in->vent_off * TEMP_PHYSICAL_MAGNITUDE_MULTIPLAYER);
   }
   break;
  case CARBUR_PAR:
   {
    CarburPar* p_in = (CarburPar*)i_values;
    p_params->ephh_hit    = p_in->ephh_hit;
    p_params->ephh_lot    = p_in->ephh_lot;
    p_params->carb_invers = p_in->carb_invers;
    p_params->epm_on_threshold = MathHelpers::Round(p_in->epm_ont * MAP_PHYSICAL_MAGNITUDE_MULTIPLAYER);
    p_params->ephh_hit_g  = p_in->ephh_hit_g;
    p_params->ephh_lot_g  = p_in->ephh_lot_g;
    p_params->shutoff_delay = MathHelpers::Round(p_in->shutoff_delay * 100); //   
   }
   break;
  case IDLREG_PAR:
   {
    IdlRegPar* p_in = (IdlRegPar*)i_values;
    p_params->idl_regul  = p_in->idl_regul;
    p_params->idling_rpm = p_in->idling_rpm;
    p_params->MINEFR     = p_in->MINEFR;
    p_params->ifac1      = MathHelpers::Round(p_in->ifac1 * ANGLE_MULTIPLAYER);
    p_params->ifac2      = MathHelpers::Round(p_in->ifac2 * ANGLE_MULTIPLAYER);
    p_params->idlreg_min_angle = MathHelpers::Round(p_in->min_angle * ANGLE_MULTIPLAYER);
    p_params->idlreg_max_angle = MathHelpers::Round(p_in->max_angle * ANGLE_MULTIPLAYER);
   }
   break;
  case ANGLES_PAR:
   {
    AnglesPar* p_in = (AnglesPar*)i_values;
    p_params->angle_corr = MathHelpers::Round(p_in->angle_corr * ANGLE_MULTIPLAYER);
    p_params->max_angle  = MathHelpers::Round(p_in->max_angle * ANGLE_MULTIPLAYER);
    p_params->min_angle  = MathHelpers::Round(p_in->min_angle * ANGLE_MULTIPLAYER);
    p_params->angle_dec_spead = MathHelpers::Round(p_in->dec_spead * ANGLE_MULTIPLAYER);
    p_params->angle_inc_spead = MathHelpers::Round(p_in->inc_spead * ANGLE_MULTIPLAYER);
    p_params->zero_adv_ang = p_in->zero_adv_ang;
   }
   break;
  case FUNSET_PAR:
   {
    FunSetPar* p_in = (FunSetPar*)i_values;
    p_params->fn_benzin = p_in->fn_benzin;
    p_params->fn_gas    = p_in->fn_gas;
    p_params->map_lower_pressure = MathHelpers::Round(p_in->map_lower_pressure * MAP_PHYSICAL_MAGNITUDE_MULTIPLAYER);
    p_params->map_upper_pressure = MathHelpers::Round(p_in->map_upper_pressure * MAP_PHYSICAL_MAGNITUDE_MULTIPLAYER);
    p_params->map_curve_offset = MathHelpers::Round(p_in->map_curve_offset / ADC_DISCRETE);
    p_params->map_curve_gradient = MathHelpers::Round(128.0f * p_in->map_curve_gradient * MAP_PHYSICAL_MAGNITUDE_MULTIPLAYER * ADC_DISCRETE);
   }
   break;
  case STARTR_PAR:
   {
    StartrPar* p_in = (StartrPar*)i_values;
    p_params->starter_off  = p_in->starter_off ;
    p_params->smap_abandon = p_in->smap_abandon;
   }
   break;
  case ADCCOR_PAR:
   {
    ADCCompenPar* p_in = (ADCCompenPar*)i_values;
    p_params->map_adc_factor      = MathHelpers::Round(p_in->map_adc_factor * 16384);
    //-------------------------------------------------------------------------
    signed long map_correction_d  = MathHelpers::Round((-p_in->map_adc_correction) / ADC_DISCRETE); //     
    p_params->map_adc_correction  = MathHelpers::Round(16384 * (0.5f - map_correction_d * p_in->map_adc_factor));
    //-------------------------------------------------------------------------

    p_params->temp_adc_factor     = MathHelpers::Round(p_in->temp_adc_factor * 16384);
    //-------------------------------------------------------------------------
    signed long temp_correction_d = MathHelpers::Round((-p_in->temp_adc_correction) / ADC_DISCRETE); //     
    p_params->temp_adc_correction = MathHelpers::Round(16384 * (0.5f - temp_correction_d * p_in->temp_adc_factor));
    //-------------------------------------------------------------------------

    p_params->ubat_adc_factor     = MathHelpers::Round(p_in->ubat_adc_factor * 16384);
    //-------------------------------------------------------------------------
    signed long ubat_correction_d = MathHelpers::Round((-p_in->ubat_adc_correction) / ADC_DISCRETE); //     
    p_params->ubat_adc_correction = MathHelpers::Round(16384 * (0.5f - ubat_correction_d * p_in->ubat_adc_factor));
    //-------------------------------------------------------------------------
   }
   break;
  case CKPS_PAR:
   {
    CKPSPar* p_in = (CKPSPar*)i_values;
    p_params->ckps_cogs_btdc  = p_in->ckps_cogs_btdc;
    p_params->ckps_ignit_cogs = p_in->ckps_ignit_cogs;
    p_params->ckps_edge_type  = p_in->ckps_edge_type;
    p_params->ref_s_edge_type  = p_in->ref_s_edge_type;
    p_params->ckps_engine_cyl = p_in->ckps_engine_cyl;
    p_params->ckps_merge_ign_outs = p_in->ckps_merge_ign_outs;
    p_params->ckps_cogs_num = p_in->ckps_cogs_num;
    p_params->ckps_miss_num = p_in->ckps_miss_num;
   }
   break;
  case KNOCK_PAR:
   {
    KnockPar* p_in = (KnockPar*)i_values;
    p_params->knock_use_knock_channel = p_in->knock_use_knock_channel;
    p_params->knock_k_wnd_begin_angle = MathHelpers::Round(p_in->knock_k_wnd_begin_angle * ANGLE_MULTIPLAYER);
    p_params->knock_k_wnd_end_angle = MathHelpers::Round(p_in->knock_k_wnd_end_angle  * ANGLE_MULTIPLAYER);
    ASSERT(p_in->knock_bpf_frequency >= 0.0f);
    p_params->knock_bpf_frequency = MathHelpers::Round(p_in->knock_bpf_frequency);
    p_params->knock_int_time_const = p_in->knock_int_time_const;

    p_params->knock_retard_step = MathHelpers::Round(p_in->knock_retard_step * ANGLE_MULTIPLAYER);
    p_params->knock_advance_step = MathHelpers::Round(p_in->knock_advance_step * ANGLE_MULTIPLAYER);
    p_params->knock_max_retard = MathHelpers::Round(p_in->knock_max_retard * ANGLE_MULTIPLAYER);
    p_params->knock_threshold = MathHelpers::Round(p_in->knock_threshold / ADC_DISCRETE);
    p_params->knock_recovery_delay = p_in->knock_recovery_delay;
   }
   break;
  case MISCEL_PAR:
   {
    MiscelPar* p_in = (MiscelPar*)i_values;

    int divisor = 0;
    for(size_t i = 0; i < SECU3IO::SECU3_ALLOWABLE_UART_DIVISORS_COUNT; ++i)
     if (SECU3IO::secu3_allowable_uart_divisors[i].first == p_in->baud_rate)
      divisor = SECU3IO::secu3_allowable_uart_divisors[i].second;
    if (0==divisor)
    {
     secu3_allowable_uart_divisors[0].second;
     ASSERT(0);
    }

    p_params->uart_divisor = divisor;
    p_params->uart_period_t_ms = p_in->period_ms / 10;
    p_params->ign_cutoff = p_in->ign_cutoff;
    p_params->ign_cutoff_thrd = p_in->ign_cutoff_thrd;
    p_params->hop_start_cogs = p_in->hop_start_cogs;
    p_params->hop_durat_cogs = p_in->hop_durat_cogs;
   }
   break;

  default:
   return false; //   
 }//switch

 return true;
}

bool CFirmwareDataMediator::GetDefParamValues(BYTE i_descriptor, void* o_values)
{
 using namespace SECU3IO;

 BYTE* p_bytes = NULL;
 params_t* p_params = NULL;
 p_bytes = mp_bytes_active;
 //    
 p_params = &((fw_data_t*)(&p_bytes[m_lip->FIRMWARE_DATA_START]))->def_param;

 //TODO: Remove these copy/paste! (conversions should be implemented in functions, as certain class)
 switch(i_descriptor)
  {
   case TEMPER_PAR:
    {
     TemperPar* p_out = (TemperPar*)o_values;
     p_out->tmp_use  = p_params->tmp_use;
     p_out->vent_pwm = p_params->vent_pwm;
     p_out->cts_use_map = p_params->cts_use_map;
     p_out->vent_on  = ((float)p_params->vent_on) / TEMP_PHYSICAL_MAGNITUDE_MULTIPLAYER;
     p_out->vent_off = ((float)p_params->vent_off) / TEMP_PHYSICAL_MAGNITUDE_MULTIPLAYER;
    }
    break;
   case CARBUR_PAR:
    {
     CarburPar* p_out = (CarburPar*)o_values;
     p_out->ephh_hit    = p_params->ephh_hit;
     p_out->ephh_lot    = p_params->ephh_lot;
     p_out->carb_invers = p_params->carb_invers;
     p_out->epm_ont =  ((float)p_params->epm_on_threshold) / MAP_PHYSICAL_MAGNITUDE_MULTIPLAYER;
     p_out->ephh_hit_g  = p_params->ephh_hit_g;
     p_out->ephh_lot_g  = p_params->ephh_lot_g;
     p_out->shutoff_delay = ((float)p_params->shutoff_delay) / 100.0f; //  
    }
    break;
   case IDLREG_PAR:
    {
     IdlRegPar* p_out = (IdlRegPar*)o_values;
     p_out->idl_regul  = p_params->idl_regul;
     p_out->idling_rpm = p_params->idling_rpm;
     p_out->MINEFR     = p_params->MINEFR;
     p_out->ifac1      = ((float)p_params->ifac1) / ANGLE_MULTIPLAYER;
     p_out->ifac2      = ((float)p_params->ifac2) / ANGLE_MULTIPLAYER;
     p_out->min_angle  = ((float)p_params->idlreg_min_angle) / ANGLE_MULTIPLAYER;
     p_out->max_angle  = ((float)p_params->idlreg_max_angle) / ANGLE_MULTIPLAYER;
    }
    break;
   case ANGLES_PAR:
    {
     AnglesPar* p_out = (AnglesPar*)o_values;
     p_out->angle_corr = ((float)p_params->angle_corr) / ANGLE_MULTIPLAYER;
     p_out->max_angle  = ((float)p_params->max_angle)  / ANGLE_MULTIPLAYER;
     p_out->min_angle  = ((float)p_params->min_angle) / ANGLE_MULTIPLAYER;
     p_out->dec_spead = ((float)p_params->angle_dec_spead)  / ANGLE_MULTIPLAYER;
     p_out->inc_spead = ((float)p_params->angle_inc_spead)  / ANGLE_MULTIPLAYER;
     p_out->zero_adv_ang = p_params->zero_adv_ang;
    }
    break;
   case FUNSET_PAR:
    {
     FunSetPar* p_out = (FunSetPar*)o_values;
     p_out->fn_benzin = p_params->fn_benzin;
     p_out->fn_gas    = p_params->fn_gas;
     p_out->map_lower_pressure = ((float)p_params->map_lower_pressure) / MAP_PHYSICAL_MAGNITUDE_MULTIPLAYER;
     p_out->map_upper_pressure = ((float)p_params->map_upper_pressure) / MAP_PHYSICAL_MAGNITUDE_MULTIPLAYER;
     p_out->map_curve_offset = ((float)p_params->map_curve_offset) * ADC_DISCRETE;
     p_out->map_curve_gradient = ((float)p_params->map_curve_gradient) / (MAP_PHYSICAL_MAGNITUDE_MULTIPLAYER * ADC_DISCRETE * 128.0f);
    }
    break;
   case STARTR_PAR:
    {
     StartrPar* p_out = (StartrPar*)o_values;
     p_out->starter_off = p_params->starter_off;
     p_out->smap_abandon = p_params->smap_abandon;
    }
    break;
   case ADCCOR_PAR:
    {
     ADCCompenPar* p_out = (ADCCompenPar*)o_values;
     p_out->map_adc_factor      = ((float)p_params->map_adc_factor) / 16384;
     //-------------------------------------------------------------------------
     p_out->map_adc_correction  = ((((float)p_params->map_adc_correction)/16384.0f) - 0.5f) / p_out->map_adc_factor;
     p_out->map_adc_correction*=ADC_DISCRETE; // 
     //-------------------------------------------------------------------------

     p_out->temp_adc_factor     = ((float)p_params->temp_adc_factor) / 16384;
     //-------------------------------------------------------------------------
     p_out->temp_adc_correction = ((((float)p_params->temp_adc_correction)/16384.0f) - 0.5f) / p_out->temp_adc_factor;
     p_out->temp_adc_correction*=ADC_DISCRETE; // 
     //-------------------------------------------------------------------------

     p_out->ubat_adc_factor     = ((float)p_params->ubat_adc_factor) / 16384;
     //-------------------------------------------------------------------------
     p_out->ubat_adc_correction = ((((float)p_params->ubat_adc_correction)/16384.0f) - 0.5f) / p_out->ubat_adc_factor;
     p_out->ubat_adc_correction*=ADC_DISCRETE; // 
     //-------------------------------------------------------------------------
    }
    break;
   case CKPS_PAR:
    {
     CKPSPar* p_out = (CKPSPar*)o_values;
     p_out->ckps_cogs_btdc  = p_params->ckps_cogs_btdc;
     p_out->ckps_ignit_cogs = p_params->ckps_ignit_cogs;
     p_out->ckps_edge_type  = p_params->ckps_edge_type;
     p_out->ref_s_edge_type  = p_params->ref_s_edge_type;
     p_out->ckps_engine_cyl = p_params->ckps_engine_cyl;
     p_out->ckps_merge_ign_outs = p_params->ckps_merge_ign_outs;
     p_out->ckps_cogs_num = p_params->ckps_cogs_num;
     p_out->ckps_miss_num = p_params->ckps_miss_num;
    }
    break;
   case KNOCK_PAR:
    {
     KnockPar* p_out = (KnockPar*)o_values;
     p_out->knock_use_knock_channel = p_params->knock_use_knock_channel;
     p_out->knock_k_wnd_begin_angle = ((float)p_params->knock_k_wnd_begin_angle) / ANGLE_MULTIPLAYER;
     p_out->knock_k_wnd_end_angle = ((float)p_params->knock_k_wnd_end_angle) / ANGLE_MULTIPLAYER;
     p_out->knock_bpf_frequency = p_params->knock_bpf_frequency;
     p_out->knock_int_time_const = p_params->knock_int_time_const;

     p_out->knock_retard_step = ((float)p_params->knock_retard_step) / ANGLE_MULTIPLAYER;
     p_out->knock_advance_step = ((float)p_params->knock_advance_step) / ANGLE_MULTIPLAYER;
     p_out->knock_max_retard = ((float)p_params->knock_max_retard) / ANGLE_MULTIPLAYER;
     p_out->knock_threshold = ((float)p_params->knock_threshold) * ADC_DISCRETE;
     p_out->knock_recovery_delay = p_params->knock_recovery_delay;
    }
    break;
   case MISCEL_PAR:
    {
     MiscelPar* p_out = (MiscelPar*)o_values;

     int baud_rate = 0;
     for(size_t i = 0; i < SECU3IO::SECU3_ALLOWABLE_UART_DIVISORS_COUNT; ++i)
     if (SECU3IO::secu3_allowable_uart_divisors[i].second == p_params->uart_divisor)
      baud_rate = SECU3IO::secu3_allowable_uart_divisors[i].first;
     ASSERT(baud_rate);

     p_out->baud_rate = baud_rate;
     p_out->period_ms = p_params->uart_period_t_ms * 10;
     p_out->ign_cutoff = p_params->ign_cutoff;
     p_out->ign_cutoff_thrd = p_params->ign_cutoff_thrd;
     p_out->hop_start_cogs = p_params->hop_start_cogs;
     p_out->hop_durat_cogs = p_params->hop_durat_cogs;
    }
    break;

   default:
    return false; //   
  }//switch

  return true;
}

//     
void CFirmwareDataMediator::GetMapsData(FWMapsDataHolder* op_fwd)
{
 std::vector<_TSTRING> names = GetFunctionsSetNames();
 int i;
 for(i = 0; i < TABLES_NUMBER; i++)
 {
  GetStartMap(i,op_fwd->maps[i].f_str);
  GetIdleMap(i,op_fwd->maps[i].f_idl);
  GetWorkMap(i,op_fwd->maps[i].f_wrk);
  GetTempMap(i,op_fwd->maps[i].f_tmp);
  op_fwd->maps[i].name = names[i];
 }
 //separate tables
 GetAttenuatorMap(op_fwd->attenuator_table);
 GetDwellCntrlMap(op_fwd->dwellcntrl_table);

 GetCTSCurveMap(op_fwd->ctscurve_table);
 op_fwd->ctscurve_vlimits[0] = GetCTSMapVoltageLimit(0);
 op_fwd->ctscurve_vlimits[1] = GetCTSMapVoltageLimit(1);

 //     (Copy table with RPM grid)
 for(i = 0; i < F_RPM_SLOTS; ++i)
  op_fwd->rpm_slots[i] = (float)work_map_rpm_slots[i];
}

void CFirmwareDataMediator::SetMapsData(const FWMapsDataHolder* ip_fwd)
{
 int i;
 for(i = 0; i < TABLES_NUMBER; i++)
 {
  SetStartMap(i,ip_fwd->maps[i].f_str);
  SetIdleMap(i,ip_fwd->maps[i].f_idl);
  SetWorkMap(i,ip_fwd->maps[i].f_wrk);
  SetTempMap(i,ip_fwd->maps[i].f_tmp);
  SetFunctionsSetName(i,ip_fwd->maps[i].name);
 }
 //separate tables
 SetAttenuatorMap(ip_fwd->attenuator_table);
 SetDwellCntrlMap(ip_fwd->dwellcntrl_table);

 SetCTSCurveMap(ip_fwd->ctscurve_table);
 SetCTSMapVoltageLimit(0, ip_fwd->ctscurve_vlimits[0]);
 SetCTSMapVoltageLimit(1, ip_fwd->ctscurve_vlimits[1]);

 //todo in the future, set values from ip_fwd->rpm_slots into the firmware here
 for(i = 0; i < F_RPM_SLOTS; ++i)
  if (ip_fwd->rpm_slots[i] != work_map_rpm_slots[i])
  {
    //-----@Add_serge_5518------
    #if defined(USE_FOR_API_BUILDER)
      ShowMessage("RPM grids from firmware and source are not equal!");
    #else
      AfxMessageBox(_T("RPM grids from firmware and source are not equal!"), MB_ICONSTOP);
    #endif
    //-end-@
    break;
  }
}

void CFirmwareDataMediator::GetAttenuatorMap(float* o_values, bool i_original /* = false */)
{
 BYTE* p_bytes = NULL;
 ASSERT(o_values);
 if (!o_values)
  return;

 if (i_original)
  p_bytes = mp_bytes_original;
 else
  p_bytes = mp_bytes_active;

 //    
 fw_data_t* p_fd = (fw_data_t*)(&p_bytes[m_lip->FIRMWARE_DATA_START]);

 for(size_t i = 0; i < KC_ATTENUATOR_LOOKUP_TABLE_SIZE; i++)
  o_values[i] = p_fd->exdata.attenuator_table[i];
}

void CFirmwareDataMediator::SetAttenuatorMap(const float* i_values)
{
 BYTE* p_bytes = NULL;
 ASSERT(i_values);
 if (!i_values)
  return;

 p_bytes = mp_bytes_active;

 //    
 fw_data_t* p_fd = (fw_data_t*)(&p_bytes[m_lip->FIRMWARE_DATA_START]);

 for(size_t i = 0; i < KC_ATTENUATOR_LOOKUP_TABLE_SIZE; i++)
 {
  ASSERT(i_values[i] >= 0.0f);
  p_fd->exdata.attenuator_table[i] = (_uchar)MathHelpers::Round(i_values[i]);
 }
}

void CFirmwareDataMediator::GetDwellCntrlMap(float* o_values, bool i_original /* = false */)
{
 BYTE* p_bytes = NULL;
 ASSERT(o_values);
 if (!o_values)
  return;

 if (i_original)
  p_bytes = mp_bytes_original;
 else
  p_bytes = mp_bytes_active;

 //    
 fw_data_t* p_fd = (fw_data_t*)(&p_bytes[m_lip->FIRMWARE_DATA_START]);

 for(size_t i = 0; i < COIL_ON_TIME_LOOKUP_TABLE_SIZE; i++)
  o_values[i] = (p_fd->exdata.coil_on_time[i] * 4.0f) / 1000.0f; //convert to ms, discrete = 4us
}

void CFirmwareDataMediator::SetDwellCntrlMap(const float* i_values)
{
 BYTE* p_bytes = NULL;
 ASSERT(i_values);
 if (!i_values)
  return;

 p_bytes = mp_bytes_active;

 //    
 fw_data_t* p_fd = (fw_data_t*)(&p_bytes[m_lip->FIRMWARE_DATA_START]);

 for(size_t i = 0; i < COIL_ON_TIME_LOOKUP_TABLE_SIZE; i++)
  p_fd->exdata.coil_on_time[i] = (_uint)MathHelpers::Round((i_values[i] * 1000.0) / 4.0);
}

void CFirmwareDataMediator::GetCTSCurveMap(float* o_values, bool i_original /* = false */)
{
 BYTE* p_bytes = NULL;
 ASSERT(o_values);
 if (!o_values)
  return;

 if (i_original)
  p_bytes = mp_bytes_original;
 else
  p_bytes = mp_bytes_active;

 //    
 fw_data_t* p_fd = (fw_data_t*)(&p_bytes[m_lip->FIRMWARE_DATA_START]);

 for(size_t i = 0; i < THERMISTOR_LOOKUP_TABLE_SIZE; i++)
  o_values[i] = (p_fd->exdata.cts_curve[i] / 4.0f);
}

void CFirmwareDataMediator::SetCTSCurveMap(const float* i_values)
{
 BYTE* p_bytes = NULL;
 ASSERT(i_values);
 if (!i_values)
  return;

 p_bytes = mp_bytes_active;

 //    
 fw_data_t* p_fd = (fw_data_t*)(&p_bytes[m_lip->FIRMWARE_DATA_START]);

 for(size_t i = 0; i < THERMISTOR_LOOKUP_TABLE_SIZE; i++)
  p_fd->exdata.cts_curve[i] = (_uint)MathHelpers::Round(i_values[i] * 4.0);
}

float CFirmwareDataMediator::GetCTSMapVoltageLimit(int i_type)
{
 //    
 fw_data_t* p_fd = (fw_data_t*)(&mp_bytes_active[m_lip->FIRMWARE_DATA_START]);
 if (0 == i_type)
  return p_fd->exdata.cts_vl_begin * ADC_DISCRETE;
 else if (1 == i_type)
  return p_fd->exdata.cts_vl_end * ADC_DISCRETE;
 else
 {
  ASSERT(0); //wrong i_type value
  return 0;
 }
}

void  CFirmwareDataMediator::SetCTSMapVoltageLimit(int i_type, float i_value)
{
 fw_data_t* p_fd = (fw_data_t*)(&mp_bytes_active[m_lip->FIRMWARE_DATA_START]);
 if (0 == i_type)
  p_fd->exdata.cts_vl_begin = MathHelpers::Round(i_value / ADC_DISCRETE);
 else if (1 == i_type)
  p_fd->exdata.cts_vl_end = MathHelpers::Round(i_value / ADC_DISCRETE);
 else
 {
  ASSERT(0); //wrong i_type value
 }
}

const PPFlashParam& CFirmwareDataMediator::GetPlatformParams(void) const
{
 return *m_fpp;
}

DWORD CFirmwareDataMediator::GetIOPlug(IOXtype type, IOPid id)
{
 if (!mp_cddata)
  return 0;
 DWORD value = 0;
 switch(type)
 {
  case IOX_INIT:
   if (id < IOP_COUNT_NUM(mp_cddata))
    value = CAST_CDDATA(mp_cddata, iorem.i_plugs[id]);
   break;
  case IOX_DATA:
   if (id < IOP_COUNT_NUM(mp_cddata))
    value = CAST_CDDATA(mp_cddata, iorem.v_plugs[id]);
   break;
 }
 return value;
}

DWORD CFirmwareDataMediator::GetIOSlot(IOXtype type, IOSid id, bool inv)
{
 if (!mp_cddata)
  return 0;
 DWORD value = 0;
 switch(type)
 {
  case IOX_INIT:
   if (id < IOS_COUNT_NUM(mp_cddata))
    value = inv ? CAST_CDDATA(mp_cddata, iorem.i_slotsi[id]) : CAST_CDDATA(mp_cddata, iorem.i_slots[id]);
   break;
  case IOX_DATA:
   if (id < IOS_COUNT_NUM(mp_cddata))
    value = inv ? CAST_CDDATA(mp_cddata, iorem.v_slotsi[id]) : CAST_CDDATA(mp_cddata, iorem.v_slots[id]);
   break;
 }
 return value;
}

DWORD CFirmwareDataMediator::GetSStub(void) const
{
 if (!mp_cddata)
  return 0;
 return CAST_CDDATA(mp_cddata, iorem.s_stub);
}

DWORD CFirmwareDataMediator::GetGStub(void) const
{
 if (!mp_cddata)
  return 0;
 return CAST_CDDATA(mp_cddata, iorem.g_stub);
}

void  CFirmwareDataMediator::SetIOPlug(IOXtype type, IOPid id, DWORD value)
{
 if (!mp_cddata)
  return;
 switch(type)
 {
  case IOX_INIT:
   if (id < IOP_COUNT_NUM(mp_cddata))
    CAST_CDDATA(mp_cddata, iorem.i_plugs[id]) = (_fnptr_t)value;
   break;
  case IOX_DATA:
   if (id < IOP_COUNT_NUM(mp_cddata))
    CAST_CDDATA(mp_cddata, iorem.v_plugs[id]) = (_fnptr_t)value;
   break;
 }
}

CFirmwareDataMediator::IORemVer CFirmwareDataMediator::GetIORemVersion(void) const
{
 ASSERT(mp_cddata);
 if (!mp_cddata)
  return IOV_V00; //error
 return (IORemVer)CAST_CDDATA(mp_cddata, iorem.version);
}

void CFirmwareDataMediator::_FindCodeData(void)
{
 //obtain pointer to cd_data_t structure, note that this pointer must be casted
 cd_data_t* p_cd = (cd_data_t*)(&mp_bytes_active[m_lip->FIRMWARE_DATA_START - sizeof(cd_data_t)]);
 //obtain pointer to fw_data_t structure
 fw_data_t* p_fd = (fw_data_t*)(&mp_bytes_active[m_lip->FIRMWARE_DATA_START]);

 _uint dataSize = p_fd->def_param.crc;

 if (0==dataSize)
  mp_cddata = NULL;
 else
  mp_cddata = p_cd;
}

bool CFirmwareDataMediator::HasCodeData(const BYTE* i_source_bytes /*= NULL*/) const
{
 if (!i_source_bytes) //current firmware (check loaded firmware)
  return (NULL != mp_cddata);
 else
 {
  //obtain pointer to fw_data_t structure
  fw_data_t* p_fd = (fw_data_t*)(&i_source_bytes[m_lip->FIRMWARE_DATA_START]);
  _uint dataSize = p_fd->def_param.crc;
  return (dataSize > 0); //has code data?
 }
}

size_t CFirmwareDataMediator::GetOnlyCodeSize(const BYTE* i_bytes) const
{
 ASSERT(i_bytes);
 //obtain pointer to fw_data_t structure
 fw_data_t* p_fd = (fw_data_t*)(&i_bytes[m_lip->FIRMWARE_DATA_START]);

 size_t sizeoffd = p_fd->def_param.crc;

 if ((sizeoffd > 0) && (sizeoffd >= m_fpp->m_only_data_size) && (sizeoffd < m_fpp->m_app_section_size))
  return m_fpp->m_app_section_size - sizeoffd;
 else //old versions of firmware
  return m_fpp->m_only_code_size;
}

void CFirmwareDataMediator::LoadCodeData(const BYTE* i_source_bytes, size_t i_srcSize, BYTE* o_destin_bytes /*= NULL*/)
{
 ASSERT(i_source_bytes);
 if (!o_destin_bytes)
  o_destin_bytes = mp_bytes_active; //use current firmware bytes as destination

 //obtain actual size of data (source and destination data)
 _uint dataSizeSrc = ((fw_data_t*)((i_source_bytes + i_srcSize)-sizeof(fw_data_t)))->def_param.crc;
 _uint dataSizeDst = ((fw_data_t*)((o_destin_bytes + m_lip->FIRMWARE_DATA_START)))->def_param.crc;
 //Check compatibility and copy data from source to destination
 if ((dataSizeSrc > 0) && dataSizeSrc && (dataSizeDst > 0) && dataSizeDst)
 { //code area data is present
  const BYTE *p_dataSrc = (i_source_bytes + i_srcSize) - dataSizeSrc;
  BYTE *p_dataDst = (o_destin_bytes + m_lip->BOOT_START) - dataSizeDst;
  size_t szSrc = _BIGEND16(*(((_uint*)p_dataSrc)-1)), szDst = _BIGEND16(*(((_uint*)p_dataDst)-1));
  cd_data_t* pSrc = (cd_data_t*)(p_dataSrc - sizeof(cd_data_t)); //must be casted!
  cd_data_t* pDst = (cd_data_t*)(p_dataDst - sizeof(cd_data_t)); //must be casted!
  if (szSrc == szDst) //compatible?
  {
   //Transfer values (don't copy it as memory block, because pointers may be different!)
   //So, we copy logical references. Not actual bits.

   for(size_t p = 0; p < IOREM_PLUGS_NUM(pSrc); ++p)
   {
    if (CAST_CDDATA(pSrc, iorem.i_plugs[p]) == 0)
     continue; //skip not implemented plugs
    size_t s = 0;
    //     -     
    bool inv = false;

    for(; s < IOREM_SLOTS_NUM(pSrc); ++s)
    {
     if (CAST_CDDATA(pSrc, iorem.i_slots[s]) == CAST_CDDATA(pSrc, iorem.i_plugs[p]))
      break;

     if (CAST_CDDATA(pSrc, iorem.i_slotsi[s]) == CAST_CDDATA(pSrc, iorem.i_plugs[p]))
     { inv = true; break; }
    }
    // V0.0       ,             
    //   union

    if ((s < IOREM_SLOTS_NUM(pSrc)) && (CAST_CDDATA(pSrc, iorem.i_plugs[p]) != CAST_CDDATA(pSrc, iorem.s_stub))) //slot?
    {
     CAST_CDDATA(pDst, iorem.i_plugs[p]) = inv ? CAST_CDDATA(pDst, iorem.i_slotsi[s]) : CAST_CDDATA(pDst, iorem.i_slots[s]);
     CAST_CDDATA(pDst, iorem.v_plugs[p]) = inv ? CAST_CDDATA(pDst, iorem.v_slotsi[s]) : CAST_CDDATA(pDst, iorem.v_slots[s]);
    }
    else if (CAST_CDDATA(pSrc, iorem.i_plugs[p]) == CAST_CDDATA(pSrc, iorem.s_stub)) //stub?
    {
     CAST_CDDATA(pDst, iorem.i_plugs[p]) = CAST_CDDATA(pDst, iorem.s_stub);
     //Use input stub if source has input stub
     if (CAST_CDDATA(pSrc, iorem.v_plugs[p]) == CAST_CDDATA(pDst, iorem.g_stub))
      CAST_CDDATA(pDst, iorem.v_plugs[p]) = CAST_CDDATA(pDst, iorem.g_stub);
     else
      CAST_CDDATA(pDst, iorem.v_plugs[p]) = CAST_CDDATA(pDst, iorem.s_stub);
    }
   }
  }
  else
  { //incompatible, then try to resolve incompatibility
   if (CAST_CDDATA(pSrc, iorem.version) == IOV_V00)
   { //   V0.0  V1.0+
    std::map<IOPid, IOPid> pidm; //for associating of plug ID from old version to plug ID from new version
    std::map<IOSid, IOSid> sidm; //for associating of slot ID from old version to slot ID from new version
    //associate plugs
    pidm.insert(std::make_pair(IOP_ECF, IOP_ECFv0));
    pidm.insert(std::make_pair(IOP_ST_BLOCK, IOP_ST_BLOCKv0));
    pidm.insert(std::make_pair(IOP_IGN_OUT3, IOP_IGN_OUT3v0));
    pidm.insert(std::make_pair(IOP_IGN_OUT4, IOP_IGN_OUT4v0));
    pidm.insert(std::make_pair(IOP_ADD_IO1, IOP_ADD_IO1v0));
    pidm.insert(std::make_pair(IOP_ADD_IO2, IOP_ADD_IO2v0));
    pidm.insert(std::make_pair(IOP_IE, IOP_IEv0));
    pidm.insert(std::make_pair(IOP_FE, IOP_FEv0));
    pidm.insert(std::make_pair(IOP_FL_PUMP, IOP_FL_PUMPv0));
    pidm.insert(std::make_pair(IOP_HALL_OUT, IOP_HALL_OUTv0));
    pidm.insert(std::make_pair(IOP_STROBE, IOP_STROBEv0));
    pidm.insert(std::make_pair(IOP_PWRRELAY, IOP_PWRRELAYv0));
    //associate slots
    sidm.insert(std::make_pair(IOS_ECFv0, IOS_ECF));
    sidm.insert(std::make_pair(IOS_ST_BLOCKv0, IOS_ST_BLOCK));
    sidm.insert(std::make_pair(IOS_IGN_OUT3v0, IOS_IGN_OUT3));
    sidm.insert(std::make_pair(IOS_IGN_OUT4v0, IOS_IGN_OUT4));
    sidm.insert(std::make_pair(IOS_ADD_IO1v0, IOS_ADD_IO1));
    sidm.insert(std::make_pair(IOS_ADD_IO2v0, IOS_ADD_IO2));
    sidm.insert(std::make_pair(IOS_IEv0, IOS_IE));
    sidm.insert(std::make_pair(IOS_FEv0, IOS_FE));

    for(size_t p = 0; p < IOREM_PLUGS_NUM(pDst); ++p)
    {
     if (CAST_CDDATA(pDst, iorem.i_plugs[p]) == 0)
      continue; //skip not implemented plugs

     //First of all detach each plug, anyway unattached plugs will be attached in CFWIORemappingController::_AttachFreeSlotsToDefaultPlugs()
     CAST_CDDATA(pDst, iorem.i_plugs[p]) = CAST_CDDATA(pDst, iorem.s_stub);

     // ,    ,        
     //      CFWIORemappingController::_CheckErrors()
     if (p == IOP_PS || p == IOP_ADD_I1 || p == IOP_ADD_I2 || p == IOP_IGN) {
      CAST_CDDATA(pDst, iorem.i_plugs[p]) = CAST_CDDATA(pDst, iorem.s_stub);
      CAST_CDDATA(pDst, iorem.v_plugs[p]) = CAST_CDDATA(pDst, iorem.g_stub);
     }
     //       ,       
     else if (p == IOP_IGN_OUT1 || p == IOP_IGN_OUT2)
     {
      CAST_CDDATA(pDst, iorem.i_plugs[p]) = CAST_CDDATA(pDst, iorem.s_stub);
      CAST_CDDATA(pDst, iorem.v_plugs[p]) = CAST_CDDATA(pDst, iorem.s_stub);
     }
     //     
     else
     {
      size_t s = 0;
      std::map<IOPid, IOPid>::const_iterator pi = pidm.find((IOPid)p);
      if (pi != pidm.end()) {
       //     -      (V0.0)
       for(; s < IOREM_SLOTS_NUM(pSrc); ++s) {
        if (CAST_CDDATA(pSrc, iorem.i_slots[s]) == CAST_CDDATA(pSrc, iorem.i_plugs[pi->second]))
         break;
       }
       if ((s < IOREM_SLOTS_NUM(pSrc)) && (CAST_CDDATA(pSrc, iorem.i_plugs[pi->second]) != CAST_CDDATA(pSrc, iorem.s_stub))) //slot?
       {
        std::map<IOSid, IOSid>::const_iterator si = sidm.find((IOSid)s);
        if (si != sidm.end()) {
         CAST_CDDATA(pDst, iorem.i_plugs[p]) = CAST_CDDATA(pDst, iorem.i_slots[si->second]);
         CAST_CDDATA(pDst, iorem.v_plugs[p]) = CAST_CDDATA(pDst, iorem.v_slots[si->second]);
        }
       }
       else if (CAST_CDDATA(pSrc, iorem.i_plugs[p]) == CAST_CDDATA(pSrc, iorem.s_stub)) //stub?
       {
        CAST_CDDATA(pDst, iorem.i_plugs[p]) = CAST_CDDATA(pDst, iorem.s_stub);
        CAST_CDDATA(pDst, iorem.v_plugs[p]) = CAST_CDDATA(pDst, iorem.s_stub);
       }
      }
     }
    }
    //If ADD_IOx are connected to default plugs, we have to connect ADD_Ix inputs to default plugs also
    if (CAST_CDDATA(pSrc, iorem.i_plugs[IOP_ADD_IO1v0]) == CAST_CDDATA(pSrc, iorem.i_slots[IOS_ADD_IO1v0]) &&
        CAST_CDDATA(pSrc, iorem.i_slots[IOS_ADD_IO1v0]) != CAST_CDDATA(pSrc, iorem.s_stub)) {
     CAST_CDDATA(pDst, iorem.i_plugs[IOP_ADD_I1]) = CAST_CDDATA(pDst, iorem.i_slots[IOS_ADD_I1]);
     CAST_CDDATA(pDst, iorem.v_plugs[IOP_ADD_I1]) = CAST_CDDATA(pDst, iorem.v_slots[IOS_ADD_I1]);
    }
    if (CAST_CDDATA(pSrc, iorem.i_plugs[IOP_ADD_IO2v0]) == CAST_CDDATA(pSrc, iorem.i_slots[IOS_ADD_IO2v0]) &&
        CAST_CDDATA(pSrc, iorem.i_plugs[IOP_ADD_IO2v0]) != CAST_CDDATA(pSrc, iorem.s_stub)) {
     CAST_CDDATA(pDst, iorem.i_plugs[IOP_ADD_I2]) = CAST_CDDATA(pDst, iorem.i_slots[IOS_ADD_I2]);
     CAST_CDDATA(pDst, iorem.v_plugs[IOP_ADD_I2]) = CAST_CDDATA(pDst, iorem.v_slots[IOS_ADD_I2]);
    }
   }
   else if (CAST_CDDATA(pDst, iorem.version) == IOV_V00)
   {//   V1.0+  V0.0
    //not implemented
   }
  }
 }
}
//-----@Add_serge_5518------
//[BCC32 Warning] FirmwareDataMediator.cpp(1432): W8059 Structure packing size has changed
//   -   -)
